@@ -75,7 +75,7 @@ else:
 __all__ = [
     'USBContext', 'USBDeviceHandle', 'USBDevice', 'hasCapability',
     'USBPoller', 'USBTransfer', 'USBTransferHelper', 'EVENT_CALLBACK_SET',
-    'USBPollerThread', 'USBEndpoint', 'USBInterfaceSetting', 'USBInterface',
+    'USBEndpoint', 'USBInterfaceSetting', 'USBInterface',
     'USBConfiguration', 'DoomedTransferError', 'getVersion', 'USBError',
 ]
 # Bind libusb1 constants and libusb1.USBError to this module, so user does not
@@ -131,6 +131,7 @@ def mayRaiseUSBError(
     ):
     if value < 0:
         __raiseUSBError(value)
+    return value
 
 try:
     namedtuple = collections.namedtuple
@@ -691,7 +692,7 @@ class USBTransfer(object):
             raise TypeError(
                 'This method cannot be called on non-iso transfers.'
             )
-        buffer_position = transfer.buffer.value
+        buffer_position = transfer.buffer
         for iso_transfer in libusb1.get_iso_packet_list(transfer_p):
             yield (
                 iso_transfer.status,
@@ -872,6 +873,7 @@ class USBTransferHelper(object):
         # Deprecated: to drop
         return self.__transfer.isSubmitted()
 
+# BBB
 class USBPollerThread(threading.Thread):
     """
     Implements libusb1 documentation about threaded, asynchronous
@@ -901,6 +903,12 @@ class USBPollerThread(threading.Thread):
           If not given, an USBError will be raised, interrupting the thread.
         """
         super(USBPollerThread, self).__init__()
+        warnings.warn(
+            'USBPollerThread causes long stalls when used with poll (it was '
+            'intended for epoll), and is generally misleading. Consider '
+            'calling looping over context.handleEvents() in a thread instead.',
+            DeprecationWarning,
+        )
         self.daemon = True
         self.__context = context
         self.__poller = poller
@@ -1298,47 +1306,54 @@ class USBDeviceHandle(object):
         mayRaiseUSBError(result)
         langid_list = cast(descriptor_string, POINTER(c_uint16))
         return [
-            libusb1.libusb_le16_to_cpu(langid_list[offset].value)
+            libusb1.libusb_le16_to_cpu(langid_list[offset])
             for offset in xrange(1, cast(descriptor_string, POINTER(c_ubyte))[0] // 2)
         ]
 
-    def getStringDescriptor(self, descriptor, lang_id):
+    def getStringDescriptor(self, descriptor, lang_id, errors='strict'):
         """
         Fetch description string for given descriptor and in given language.
         Use getSupportedLanguageList to know which languages are available.
-        Return value is an unicode string.
+        Return value is a unicode string.
         Return None if there is no such descriptor on device.
         """
-        descriptor_string, _ = create_binary_buffer(STRING_LENGTH)
+        descriptor_string = bytearray(STRING_LENGTH)
         try:
-            mayRaiseUSBError(libusb1.libusb_get_string_descriptor(
-                self.__handle, descriptor, lang_id, descriptor_string,
-                sizeof(descriptor_string),
+            received = mayRaiseUSBError(libusb1.libusb_get_string_descriptor(
+                self.__handle, descriptor, lang_id,
+                create_binary_buffer(descriptor_string)[0],
+                STRING_LENGTH,
             ))
         # pylint: disable=undefined-variable
         except USBErrorNotFound:
             # pylint: enable=undefined-variable
             return None
-        return descriptor_string.value.decode('UTF-16-LE')
+        if received < 2 or descriptor_string[1] != DT_STRING:
+            raise ValueError('Invalid string descriptor')
+        return descriptor_string[2:min(
+            received,
+            descriptor_string[0],
+        )].decode('UTF-16-LE', errors=errors)
 
-    def getASCIIStringDescriptor(self, descriptor):
+    def getASCIIStringDescriptor(self, descriptor, errors='strict'):
         """
         Fetch description string for given descriptor in first available
         language.
-        Return value is an ASCII string.
+        Return value is a unicode string.
         Return None if there is no such descriptor on device.
         """
-        descriptor_string, _ = create_binary_buffer(STRING_LENGTH)
+        descriptor_string = bytearray(STRING_LENGTH)
         try:
-            mayRaiseUSBError(libusb1.libusb_get_string_descriptor_ascii(
-                self.__handle, descriptor, descriptor_string,
-                sizeof(descriptor_string),
+            received = mayRaiseUSBError(libusb1.libusb_get_string_descriptor_ascii(
+                self.__handle, descriptor,
+                create_binary_buffer(descriptor_string)[0],
+                STRING_LENGTH,
             ))
         # pylint: disable=undefined-variable
         except USBErrorNotFound:
             # pylint: enable=undefined-variable
             return None
-        return descriptor_string.value.decode('ASCII')
+        return descriptor_string[:received].decode('ASCII', errors=errors)
 
     # Sync I/O
 
@@ -2328,7 +2343,7 @@ class USBContext(object):
     @_validContext
     def setDebug(self, level):
         """
-        Set debugging level.
+        Set debugging level. See LOG_LEVEL_* constants.
         Note: depending on libusb compilation settings, this might have no
         effect.
         """
