@@ -17,19 +17,22 @@
 # pylint: disable=invalid-name, missing-docstring, too-many-public-methods
 import unittest
 import sys
+import itertools
 import select
 import threading
 import usb1
 import libusb1
 from ctypes import pointer
 
+buff_len = 1024
+buffer_base = [x % 256 for x in range(buff_len)]
 if sys.version_info[0] == 3:
-    buff = bytes([0, 0xff])
-    other_buff = bytes((ord(x) for x in 'foo'))
+    buff = bytes(buffer_base)
+    other_buff = bytes(reversed(buffer_base))
 else:
-    buff = '\x00\xff'
-    other_buff = 'foo'
-buff_len = 2
+    buff = ''.join(chr(x) for x in buffer_base)
+    other_buff = ''.join(chr(x) for x in reversed(buffer_base))
+bytearray_buff = bytearray(buffer_base)
 
 class USBContext(usb1.USBContext):
     def open(self):
@@ -115,8 +118,7 @@ class USBTransferTests(unittest.TestCase):
         # No callback
         transfer.setControl(request_type, request, value, index, buff)
 
-    def _testSetBulkOrInterrupt(self, setter_id):
-        transfer = self.getTransfer()
+    def _testTransferSetter(self, transfer, setter_id):
         endpoint = 0x81
         def callback(transfer):
             pass
@@ -130,6 +132,8 @@ class USBTransferTests(unittest.TestCase):
         self.assertEqual(buff, transfer.getBuffer())
         transfer.setBuffer(other_buff)
         self.assertEqual(other_buff, transfer.getBuffer())
+        transfer.setBuffer(bytearray_buff)
+        self.assertEqual(bytearray_buff, transfer.getBuffer())
         transfer.setBuffer(buff_len)
         self.assertEqual(buff_len, len(transfer.getBuffer()))
         # All provided, buffer length variant
@@ -148,14 +152,45 @@ class USBTransferTests(unittest.TestCase):
         Simplest test: feed some data, must not raise.
         Also, test setBuffer/getBuffer.
         """
-        self._testSetBulkOrInterrupt('setBulk')
+        self._testTransferSetter(self.getTransfer(), 'setBulk')
 
     def testSetInterrupt(self):
         """
         Simplest test: feed some data, must not raise.
         Also, test setBuffer/getBuffer.
         """
-        self._testSetBulkOrInterrupt('setInterrupt')
+        self._testTransferSetter(self.getTransfer(), 'setInterrupt')
+
+    def testSetIsochronous(self):
+        """
+        Simplest test: feed some data, must not raise.
+        Also, test setBuffer/getBuffer/getISOBufferList/iterISO.
+        """
+        iso_transfer_count = 16
+        transfer = self.getTransfer(iso_transfer_count)
+        self._testTransferSetter(transfer, 'setIsochronous')
+        # Returns whole buffers
+        self.assertEqual(
+            bytearray(itertools.chain(*transfer.getISOBufferList())),
+            buff,
+        )
+        # Returns actually transfered data, so here nothing
+        self.assertEqual(bytearray(
+            itertools.chain(*[x for _, x in transfer.iterISO()])),
+            bytearray(),
+        )
+        # Fake reception of whole transfers
+        c_transfer = getattr(
+          transfer,
+          '_' + transfer.__class__.__name__ + '__transfer'
+        )
+        for iso_metadata in libusb1.get_iso_packet_list(c_transfer):
+            iso_metadata.actual_length = iso_metadata.length
+        # Now iterISO returns everythig
+        self.assertEqual(bytearray(
+            itertools.chain(*[x for _, x in transfer.iterISO()])),
+            buff,
+        )
 
     def testSetGetCallback(self):
         transfer = self.getTransfer()
